using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman10 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman10() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman10(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman10(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman I.12.2 q1*q2*r/(4*pi*epsilon*r**3) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new[] {"q1", "q2", "epsilon", "r", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q1", "q2", "epsilon", "r"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q1      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q2      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var F = new List<double>();

      data.Add(q1);
      data.Add(q2);
      data.Add(epsilon);
      data.Add(r);
      data.Add(F);

      for (var i = 0; i < q1.Count; i++) {
        var res = q1[i] * q2[i] * r[i] / (4 * Math.PI * epsilon[i] * Math.Pow(r[i], 3));
        F.Add(res);
      }

      return data;
    }
  }
}