using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman32 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman32() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman32(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman32(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman I.32.5 q**2*a**2/(6*pi*epsilon*c**3) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "Pwr"; } }
    protected override string[] VariableNames { get { return new[] {"q", "a", "epsilon", "c", "Pwr"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "a", "epsilon", "c"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var a       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var c       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Pwr = new List<double>();

      data.Add(q);
      data.Add(a);
      data.Add(epsilon);
      data.Add(c);
      data.Add(Pwr);

      for (var i = 0; i < q.Count; i++) {
        var res = Math.Pow(q[i], 2) * Math.Pow(a[i], 2) / (6 * Math.PI * epsilon[i] * Math.Pow(c[i], 3));
        Pwr.Add(res);
      }

      return data;
    }
  }
}