using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman89 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman89() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman89(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman89(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman III.8.54 sin(E_n*t/(h/(2*pi)))**2 | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "prob"; } }
    protected override string[] VariableNames { get { return new[] {"E_n", "t", "h", "prob"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"E_n", "t", "h"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var E_n  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var t    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 4).ToList();

      var prob = new List<double>();

      data.Add(E_n);
      data.Add(t);
      data.Add(h);
      data.Add(prob);

      for (var i = 0; i < E_n.Count; i++) {
        var res = Math.Pow(Math.Sin(E_n[i] * t[i] / (h[i] / (2 * Math.PI))), 2);
        prob.Add(res);
      }

      return data;
    }
  }
}